@@ -1,9 +1,11 @@
 from django.db import models
-from django.utils import timezone as datetime
 from django.contrib.contenttypes import generic
 from django.contrib.contenttypes.models import ContentType
 from django.utils.translation import ugettext_lazy as _
 from django.db.models.signals import post_save
+from django.utils import timezone as datetime
+
+from dateutil.relativedelta import relativedelta
 
 from .fields import ResourceField
 from .helpers import get_payer
@@ -14,6 +16,7 @@ from .api import handler
 from . import settings
 
 from leetchi import resources
+from leetchi.base import DoesNotExist
 
 
 class ApiModel(models.Model):
@@ -75,6 +78,31 @@ class BaseLeetchi(ApiModel):
                               self.object_id)
 
 
+class ContributionQuerySet(models.query.QuerySet):
+    def success(self):
+        return self.filter(is_success=True)
+
+    def refundable(self):
+        now = datetime.now()
+
+        qs = self.filter(creation_date__gt=now - relativedelta(months=10))
+
+        qs = qs.filter(models.Q(card_expiration_date__isnull=True) | models.Q(card_expiration_date__gt=now))
+
+        return qs
+
+
+class ContributionManager(models.Manager):
+    def get_query_set(self):
+        return ContributionQuerySet(self.model)
+
+    def success(self):
+        return self.get_query_set().success()
+
+    def refundable(self):
+        return self.get_query_set().refundable()
+
+
 class Contribution(BaseLeetchi):
     TYPE_PAYLINE = 1
     TYPE_OGONE = 2
@@ -96,6 +124,10 @@ class Contribution(BaseLeetchi):
                                             default=TYPE_PAYLINE,
                                             verbose_name=_('Type'),
                                             db_index=True)
+    card_expiration_date = models.DateField(null=True)
+    card_number = models.CharField(max_length=100, null=True)
+
+    objects = ContributionManager()
 
     class Meta:
         db_table = 'leetchi_contribution'
@@ -153,12 +185,30 @@ class Contribution(BaseLeetchi):
             self.is_success = False
             self.is_completed = True
 
+        try:
+            payment_card = self.contribution.detail_payment_card
+        except DoesNotExist:
+            pass
+        else:
+            expiration_date = payment_card.expiration_date_converted
+
+            if expiration_date:
+                self.card_expiration_date = expiration_date
+
+            self.card_number = payment_card.number
+
         if commit:
-            update_fields(self, fields=('is_success', 'is_completed', ))
+            self.save()
 
     def is_error(self):
         return not self.is_success and self.is_completed
 
+    def sync(self, *args, **kwargs):
+        result = super(Contribution, self).sync(*args, **kwargs)
+
+        if result:
+            self.sync_status(commit=False)
+
 
 class Transfer(BaseLeetchi):
     transfer = ResourceField(resources.Transfer, null=True)
@@ -258,6 +308,12 @@ class Refund(BaseLeetchi):
         if commit and changed:
             update_fields(self, fields=('is_success', 'is_completed',))
 
+    def sync(self, *args, **kwargs):
+        result = super(Refund, self).sync(*args, **kwargs)
+
+        if result:
+            self.sync_status(commit=False)
+
 
 class Beneficiary(ApiModel):
     user = models.ForeignKey(User)
@@ -438,3 +494,17 @@ class StrongAuthentication(ApiModel):
             'user': get_payer(self.user),
             'beneficiary_id': beneficiary
         }
+
+
+def get_pending_amount(user):
+    result = (Withdrawal.objects.filter(user=user, is_completed=False)
+              .aggregate(amount=models.Sum('amount')))
+
+    amount = result.get('amount', 0) or 0
+
+    result = (Refund.objects.filter(user=user, is_completed=False)
+              .aggregate(amount=models.Sum('contribution__amount')))
+
+    amount += result.get('amount', 0) or 0
+
+    return amount
