@@ -13,6 +13,10 @@ urlpatterns = patterns(
         staff_member_required(views.ContributionDetailView.as_view()),
         name='leetchi_contribution_detail'),
 
+    url(r'^user/detail/(?P<user_id>\d+)/$',
+        staff_member_required(views.UserDetailView.as_view()),
+        name='leetchi_user_detail'),
+
     url(r'^refund/detail/(?P<refund_id>\d+)/$',
         staff_member_required(views.RefundDetailView.as_view()),
         name='leetchi_refund_detail'),
